/*
 * Copyright (C) 2014 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.dev.lavin.bcard.blueetooth

import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.util.Log
import android.view.MenuItem
import android.view.View
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.DeviceListAdapter
import com.dev.lavin.domain2.model.Device
import kotlinx.android.synthetic.main.activity_device_list.*

/**
 * This Activity appears as a dialog. It lists any paired devices and
 * devices detected in the area after discovery. When a device is chosen
 * by the user, the MAC address of the device is sent back to the parent
 * Activity in the result Intent.
 */
class DeviceListActivity : AppCompatActivity(), DeviceListAdapter.OnDeviceSelectedListener {

    /**
     * Member fields
     */
    private var mBtAdapter: BluetoothAdapter? = null


    /**
     * Newly discovered devices
     */
    var pairedDeviceList: MutableList<Device> = ArrayList()
    var pairedDeviceListAdapter: DeviceListAdapter? = null

    /**
     * Newly discovered devices
     */
    var deviceList: MutableList<Device> = ArrayList()
    var newDeviceListAdapter: DeviceListAdapter? = null

    /**
     * The on-click listener for all devices in the ListViews
     */
    override fun onDeviceSelected(device: Device) {
        mBtAdapter!!.cancelDiscovery()
        // Create the result Intent and include the MAC address
        val intent = Intent()
        intent.putExtra(EXTRA_DEVICE_ADDRESS, device.addres)
        intent.putExtra(EXTRA_DEVICE_NAME, device.name)
        // Set result and finish this Activity
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    /**
     * The BroadcastReceiver that listens for discovered devices and changes the title when
     * discovery is finished
     */
    private val mReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            // When discovery finds a device
            if (BluetoothDevice.ACTION_FOUND == action) {
                // Get the BluetoothDevice object from the Intent
                val device = intent.getParcelableExtra<BluetoothDevice>(BluetoothDevice.EXTRA_DEVICE)
                // If it's already paired, skip it, because it's been listed already
                if (device.bondState != BluetoothDevice.BOND_BONDED) {
                    val device = Device(device.name, device.address)
                    if (!checkIfExists(device, deviceList)) {
                        deviceList.add(device)
                        newDeviceListAdapter?.notifyDataSetChanged()
                    }
                }
                // When discovery is finished, change the Activity title
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED == action) {
                progressBar?.visibility = View.GONE
                setTitle(R.string.select_device)
                if (newDeviceListAdapter?.itemCount == 0) {
                    NoDevices?.visibility = View.VISIBLE
                }
            }
        }
    }

    private fun checkIfExists(device: Device, currentList: MutableList<Device>): Boolean {
        for (item: Device in currentList) {
            if (item.name == device.name && item.addres == device.addres) {
                return true
            }
        }
        return false
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Setup the window
        setContentView(R.layout.activity_device_list)

        //Setup Toolbar
        setSupportActionBar(DeviceListToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)

        // Set result CANCELED in case the user backs out
        setResult(Activity.RESULT_CANCELED)

        // Initialize array adapters. One for already paired devices and
        // one for newly discovered devices
        newDeviceListAdapter = DeviceListAdapter(deviceList, this)
        pairedDeviceListAdapter = DeviceListAdapter(pairedDeviceList, this)

        // Find and set up the RecyclerView for paired devices
        PairedDevices.layoutManager = LinearLayoutManager(this)
        PairedDevices.adapter = pairedDeviceListAdapter
        PairedDevices.setHasFixedSize(true)

        // Find and set up the RecyclerView for newly discovered devices
        NewDevices.layoutManager = LinearLayoutManager(this)
        NewDevices?.adapter = newDeviceListAdapter
        NewDevices.setHasFixedSize(true)

        // Register for broadcasts when a device is discovered
        var filter = IntentFilter(BluetoothDevice.ACTION_FOUND)
        this.registerReceiver(mReceiver, filter)

        // Register for broadcasts when discovery has finished
        filter = IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED)
        this.registerReceiver(mReceiver, filter)

        // Get the local Bluetooth adapter
        mBtAdapter = BluetoothAdapter.getDefaultAdapter()

        doDiscovery()

        // Get a set of currently paired devices
        val pairedDevices = mBtAdapter!!.bondedDevices

        // If there are paired devices, add each one to the ArrayAdapter
        if (pairedDevices.size > 0) {
            TitlePairedDevices?.visibility = View.VISIBLE
            for (device in pairedDevices) {
                pairedDeviceList.add(Device(device.name, device.address))
                pairedDeviceListAdapter?.notifyDataSetChanged()
            }
        } else {
            NoDevicesPaired?.visibility = View.VISIBLE
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onDestroy() {
        super.onDestroy()

        // Make sure we're not doing discovery anymore
        if (mBtAdapter != null) {
            mBtAdapter!!.cancelDiscovery()
        }

        // Unregister broadcast listeners
        this.unregisterReceiver(mReceiver)
    }

    /**
     * Start device discover with the BluetoothAdapter
     */
    private fun doDiscovery() {
        Log.d(TAG, "doDiscovery()")

        // Indicate scanning in the title
        progressBar?.visibility = View.VISIBLE

        // Turn on sub-title for new devices
        TitleNewDevices?.visibility = View.VISIBLE

        // If we're already discovering, stop it
        if (mBtAdapter?.isDiscovering!!) {
            mBtAdapter?.cancelDiscovery()
        }

        // Request discover from BluetoothAdapter
        mBtAdapter!!.startDiscovery()
    }

    companion object {

        /**
         * Tag for Log
         */
        private const val TAG = "DeviceListActivity"

        /**
         * Return Intent extra
         */
        var EXTRA_DEVICE_ADDRESS = "device_address"
        var EXTRA_DEVICE_NAME = "device_name"
    }

}
