package com.dev.lavin.data.services


import android.app.Activity
import android.net.Uri
import android.util.Log
import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.data.entities.event.EventEntity
import com.dev.lavin.data.entities.mappers.EventToEventEntityMapper
import com.dev.lavin.domain2.boundaries.EventsRepoInterface
import com.dev.lavin.domain2.model.Event
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.DocumentChange
import com.google.firebase.firestore.FieldValue
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.firestore.Query
import com.google.firebase.storage.FirebaseStorage
import java.sql.Timestamp
import java.util.*
import javax.inject.Inject


/**
 * Created by Jorge Lavin on 16/04/18.
 */

class EventsService(baseRepositoryComponent: BaseRepositoryComponent, private val activty: Activity) : EventsRepoInterface {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var mFireStore: FirebaseFirestore

    @Inject
    lateinit var mFireStorage: FirebaseStorage

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun getEvents(eventsResponseListener: EventsRepoInterface.EventsResponseListener) {
        val query = mFireStore.collection("Events").orderBy("timestamp", Query.Direction.DESCENDING)
        query.addSnapshotListener { documentSnapshots, e ->
            // Handle errors
            if (e != null) {
                eventsResponseListener.onEventsFailure(e.message!!)
            } else {
                if (!documentSnapshots!!.isEmpty) {
                    val eventsList: MutableMap<String, MutableList<Event>> = mutableMapOf()
                    val eventsListAdded: MutableList<Event> = ArrayList()
                    val eventsListModified: MutableList<Event> = ArrayList()
                    for (doc in documentSnapshots.documentChanges) {
                        if (doc.type == DocumentChange.Type.ADDED) {
                            val eventId = doc.document.id
                            val eventEntity: EventEntity = doc.document.toObject(EventEntity::class.java).withId(eventId)
                            val mapper = EventToEventEntityMapper()
                            eventsListAdded.add(mapper.reverseMap(eventEntity))
                        }
                        if (doc.type == DocumentChange.Type.MODIFIED) {
                            val eventId = doc.document.id
                            val eventEntity: EventEntity = doc.document.toObject(EventEntity::class.java).withId(eventId)
                            val mapper = EventToEventEntityMapper()
                            eventsListModified.add(mapper.reverseMap(eventEntity))
                        }
                    }
                    eventsList["added"] = eventsListAdded
                    eventsList["modified"] = eventsListModified
                    eventsResponseListener.onEventsSuccesResponse(eventsList)
                } else {
                    eventsResponseListener.onEventsEmptyResponse()
                }
            }
        }
    }

    override fun getEventDetail(id: String, eventDetailResponseListener: EventsRepoInterface.EventDetailResponseListener) {
        mFireStore.collection("Events").document(id).get().addOnCompleteListener { info ->
            if (info.isSuccessful) {
                if (!info.result.exists()) {
                    eventDetailResponseListener.onEventNotExistResponse()
                } else {
                    val eventId = info.result.id
                    val cardEntity: EventEntity = info.result.toObject(EventEntity::class.java)!!.withId(eventId)
                    val mapper = EventToEventEntityMapper()
                    eventDetailResponseListener.onEventDetailSuccesResponse(mapper.reverseMap(cardEntity))
                }
            } else {
                if (info.exception != null) {
                    val errorMessage = info.exception!!.message
                    if (errorMessage != null) {
                        eventDetailResponseListener.onEventDetaiFailure(errorMessage)
                    }
                }
            }
        }
    }

    override fun createEvent(name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri, eventsCreateResponseListener: EventsRepoInterface.EventsCreateResponseListener) {
        val myStorageReference = mFireStorage.getReference("events_images")
        val uploadReference = myStorageReference.child(name + Timestamp(System.currentTimeMillis()))
        val uploadEventImage = uploadReference.putFile(url)
        uploadEventImage.addOnCompleteListener { uploadTask ->
            if (uploadTask.isSuccessful) {
                val downloadUrl = uploadTask.result.downloadUrl
                val eventMap: MutableMap<String, Any> = mutableMapOf()
                eventMap["name"] = name
                eventMap["event_image_url"] = downloadUrl.toString()
                if (mAuth.currentUser != null) {
                    eventMap["user_id"] = mAuth.currentUser!!.uid
                }
                eventMap["time"] = time
                eventMap["timestamp"] = date
                eventMap["location"] = location
                eventMap["category"] = category
                eventMap["description"] = description
                eventMap["created"] = FieldValue.serverTimestamp()
                mFireStore.collection("Events").add(eventMap).addOnCompleteListener { setTask ->
                    if (setTask.isSuccessful) {
                        eventsCreateResponseListener.onEventsCreateSuccesResponse()
                    } else {
                        if (setTask.exception != null) {
                            val errorMessage = setTask.exception!!.message
                            if (errorMessage != null) {
                                eventsCreateResponseListener.onEventsCreateFailure(errorMessage)
                            }
                        }
                    }
                }
            } else {
                if (uploadTask.exception != null) {
                    val errorMessage = uploadTask.exception!!.message
                    if (errorMessage != null) {
                        eventsCreateResponseListener.onEventsCreateFailure(errorMessage)
                    }
                }
            }
        }
    }

    override fun editEvent(id: String, name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri?, eventsEditResponseListener: EventsRepoInterface.EventsEditResponseListener) {
        if (url != null) {
            val myStorageReference = mFireStorage.getReference("events_images")
            val uploadReference = myStorageReference.child(name + Timestamp(System.currentTimeMillis()))
            val uploadEditEventImage = uploadReference.putFile(url)
            uploadEditEventImage.addOnCompleteListener { uploadTask ->
                if (uploadTask.isSuccessful) {
                    val downloadUrl = uploadTask.result.downloadUrl
                    editEventValues(id, name, date, time, location, category, description, downloadUrl, eventsEditResponseListener)
                } else {
                    if (uploadTask.exception != null) {
                        val errorMessage = uploadTask.exception!!.message
                        if (errorMessage != null) {
                            eventsEditResponseListener.onEventsEditFailure(errorMessage)
                        }
                    }
                }
            }
        } else {
            editEventValues(id, name, date, time, location, category, description, url, eventsEditResponseListener)
        }
    }

    private fun editEventValues(id: String, name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri?, eventsEditResponseListener: EventsRepoInterface.EventsEditResponseListener) {
        val eventMap: MutableMap<String, Any> = mutableMapOf()
        val eventDocRef = mFireStore.collection("Events").document(id)
        mFireStore.runTransaction { transaction ->
            val eventEntity = transaction.get(eventDocRef).toObject(EventEntity::class.java)
            if (eventEntity != null) {

                if (url != null) {
                    eventMap["event_image_url"] = url.toString()
                }
                eventMap["name"] = name
                eventMap["time"] = time
                eventMap["timestamp"] = date
                eventMap["location"] = location
                eventMap["category"] = category
                eventMap["description"] = description
                transaction.update(eventDocRef, eventMap)

            }
            null
        }.addOnSuccessListener {
            Log.d("updateEvent", "Transaction success!")
            eventsEditResponseListener.onEventsEditSuccesResponse(eventMap)
        }.addOnFailureListener { e ->
            Log.w("updateEvent", "Transaction failure.", e)
            eventsEditResponseListener.onEventsEditFailure(e.message!!)
        }
    }
}
